package de.grogra.ext.x3d.importation;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import org.apache.xmlbeans.XmlException;
import de.grogra.ext.x3d.X3DImport;
import de.grogra.ext.x3d.util.Util;
import de.grogra.ext.x3d.xmlbeans.X3DDocument;
import de.grogra.ext.x3d.xmlbeans.ExternProtoDeclareDocument.ExternProtoDeclare;
import de.grogra.ext.x3d.xmlbeans.ProtoBodyDocument.ProtoBody;
import de.grogra.ext.x3d.xmlbeans.ProtoDeclareDocument.ProtoDeclare;
import de.grogra.ext.x3d.xmlbeans.SceneDocument.Scene;

public class ExternProtoDeclareImport extends ProtoDeclareImport {

	@SuppressWarnings("unchecked")
	public static void createInstance(ExternProtoDeclare externProtoDeclare) {
		ExternProtoDeclareImport epdi = new ExternProtoDeclareImport();
		
		HashMap<String, ProtoDeclareImport> protoMap = X3DImport.getTheImport().getCurrentParser().getProtoMap();
		
		boolean loadedFromWeb = false;
		File f = null;
		
		URL url = null;
		
		List<String> urls = externProtoDeclare.getUrl();
		String tmpUrls = urls.toString();
		urls = Util.splitStringToListOfStrings(tmpUrls);
		
		String protoName = null; 
		int urlCount = urls.size();
		
		for (int urlIndex = 0; urlIndex < urlCount; urlIndex++) {
			try {
				String path = urls.get(urlIndex);	
			
				if (path.startsWith("\"") && path.endsWith("\""))
					path = path.substring(1, path.length()-1);
				
				int hashMarkPos = path.indexOf("#");
				int urlLength = path.length();
				
				if (hashMarkPos == -1)
					path = path.substring(0, urlLength);
				else {
					protoName = path.substring(hashMarkPos+1, urlLength);
					path = path.substring(0, hashMarkPos);
				}	
			
				if (path.toLowerCase().startsWith("http://")) {
					String filename = path.substring(path.lastIndexOf("/") + 1, path.lastIndexOf("."));
					String fileext = path.substring(path.lastIndexOf("."), path.length());
					
					f = File.createTempFile(filename, fileext);
					
					// image path is a url
					url = new URL(path);
					
					// download x3d file to temp directory, then link it to groimp project and delete file
					InputStream is = url.openStream();
					FileOutputStream os = new FileOutputStream(f);
					
					byte[] buffer = new byte[0xFFFF];
					for (int len; (len = is.read(buffer)) != -1; )
						os.write(buffer, 0, len);
					is.close();
					os.close();

					url = f.toURI().toURL();
					loadedFromWeb = true;

				}
				else {
					// path is a file
					if (path.startsWith("/") || (path.charAt(1) == ':')) {
						// absolute path - nothing to do
					}
					else {
						// relative path
						File x3dfile = X3DImport.getTheImport().getCurrentParser().getFile();
						path = Util.getRealPath(x3dfile) + path;
					}
					f = new File(path);
					url  = f.toURI().toURL();
					
					// to test if url is correct (x3d file exists)
					Object testContent = url.getContent();
					if (testContent == null)
						continue;
					loadedFromWeb = false;
				}
				
				X3DDocument x3dDocument = null;
				try {
					x3dDocument = X3DDocument.Factory.parse(f);
				} catch (XmlException e) {e.printStackTrace();return;}
				  catch (IOException e) {e.printStackTrace();return;}
				
				Scene scene = x3dDocument.getX3D().getScene();
				ProtoDeclare[] protos = scene.getProtoDeclareArray();
				
				ProtoDeclare protoDeclare = null;
				
				if (protoName == null) {
					protoDeclare = protos[0];
				}
				else {
					for (ProtoDeclare proto : protos) {
						if (proto.getName().equals(protoName)) {
							protoDeclare = proto;
							break;
						}
					}
				}
				
				if (protoDeclare == null)
					continue;
				
				ProtoBody protoBody = protoDeclare.getProtoBody();
				epdi.protoBody = protoBody;
				
				protoMap.put(externProtoDeclare.getName(), epdi);
				
				break;
				
			} catch (MalformedURLException e) {} catch (IOException e) {} finally {
				if (loadedFromWeb && f != null) {
				// delete temporary x3d file
					f.delete();
				}	
			}
		}
		
	}
	
}
